import React, { ReactNode, useContext, useState } from "react"
import { useEffect } from "react"
import { useJwt } from "react-jwt";

const TOKEN_KEY = 'token';

export type AuthContextProps = {
  login?: string,
  setToken: (name: string) => void,
  logout: () => void,
}

type ContextProps = {
  children: ReactNode
}

const authContext = React.createContext<AuthContextProps | null>(null);

export function ProvideAuth(props: ContextProps) {
  const auth = useProvideAuth();
  return <authContext.Provider value={auth}>{props.children}</authContext.Provider>
}

export const useAuth = () => {
  return useContext(authContext);
}

function useProvideAuth(): AuthContextProps {
  const token = localStorage.getItem(TOKEN_KEY);
  const [loginName, setLoginName] = useState<string | undefined>();
  let decodedToken, isExpired;
  if (token) {
    const payload = useJwt(token);
    decodedToken = payload?.decodedToken;
    isExpired = payload?.isExpired
  }

  useEffect(() => {
    if (token) {
      if (decodedToken && !isExpired) {
        doSetToken(token);
        setLoginName((decodedToken as any).login);
      }
    }
  }, [decodedToken, isExpired])

  useEffect(() => {
    if (token) {
      localStorage.setItem(TOKEN_KEY, token);
    } else {
      localStorage.removeItem(TOKEN_KEY);
    }
  }, [token]);

  function doSetToken(token: string) {

    if (!decodedToken || !((decodedToken as any).login)) {
      throw Error("Chyba dekódování tokenu");
    }
    if (isExpired) {
      throw Error("Platnost tokenu vypršela");
    }
    setLoginName((decodedToken as any).login);
  }

  function logout() {
    setLoginName(undefined);
  }

  return {
    login: loginName,
    setToken: doSetToken,
    logout,
  }
}
