import jwt from 'jsonwebtoken';

/**
 * Vygeneruje a vrátí podepsaný JWT token pro daný login.
 * 
 * @param login přihlašovací jméno uživatele
 * @returns JWT token
 */
export function generateToken(login: string): string {
    if (!process.env.JWT_TOKEN) {
        throw Error("Není vyplněna proměnná prostředí JWT_TOKEN");
    }
    if (process.env.JWT_TOKEN.length < 32) {
        throw Error("Proměnná prostředí JWT_TOKEN musí být minimálně 32 znaků");
    }
    return jwt.sign({ login }, process.env.JWT_TOKEN);
}

/**
 * Vrátí true, pokud je předaný JWT token platný.
 * 
 * @param token JWT token
 */
export function verify(token: string): boolean {
    if (!process.env.JWT_TOKEN) {
        throw Error("Není vyplněna proměnná prostředí JWT_TOKEN");
    }
    try {
        jwt.verify(token, process.env.JWT_TOKEN);
        return true;
    } catch (err) {
        return false;
    }
}

/**
 * Vrátí login z daného JWT tokenu, pokud je token platný.
 * 
 * @param token JWT token
 */
export function getLogin(token: string): string {
    if (!process.env.JWT_TOKEN) {
        throw Error("Není vyplněna proměnná prostředí JWT_TOKEN");
    }
    const payload: any = jwt.verify(token, process.env.JWT_TOKEN);
    return payload.login;
}